@@ -1,5 +1,4 @@
 /* Download dialogs */
-/* $Id: download.c,v 1.67.4.4 2005/05/01 22:16:08 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -12,14 +11,21 @@
 #include "elinks.h"
 
 #include "bfu/dialog.h"
+#include "bfu/hierbox.h"
 #include "dialogs/download.h"
 #include "dialogs/menu.h"
+#include "dialogs/progress.h"
 #include "dialogs/status.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/select.h"
+#include "main/object.h"
+#include "main/select.h"
+#include "network/connection.h"
+#include "network/progress.h"
+#include "protocol/bittorrent/dialogs.h"
+#include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/download.h"
-#include "sched/session.h"
+#include "session/download.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/terminal.h"
 #include "util/color.h"
@@ -27,9 +33,8 @@
 #include "util/error.h"
 #include "util/memlist.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
-#include "util/ttime.h"
+#include "util/time.h"
 
 
 static void
@@ -50,47 +55,53 @@ do_abort_download(struct file_download *file_download)
 	}
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 dlg_set_notify(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct file_download *file_download = dlg_data->dlg->udata;
 
 	file_download->notify = 1;
+#if CONFIG_BITTORRENT
+	if (file_download->uri->protocol == PROTOCOL_BITTORRENT)
+		set_bittorrent_notify_on_completion(&file_download->download,
+						    file_download->term);
+#endif
 	undisplay_download(file_download);
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 dlg_abort_download(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct file_download *file_download = dlg_data->dlg->udata;
 
 	object_unlock(file_download);
-	register_bottom_half((void (*)(void *)) do_abort_download,
-			     file_download);
+	register_bottom_half(do_abort_download, file_download);
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_delete_button(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct file_download *file_download = dlg_data->dlg->udata;
 
 	file_download->delete = 1;
+#if CONFIG_BITTORRENT
+	if (file_download->uri->protocol == PROTOCOL_BITTORRENT)
+		set_bittorrent_files_for_deletion(&file_download->download);
+#endif
 	object_unlock(file_download);
-	register_bottom_half((void (*)(void *)) do_abort_download,
-			     file_download);
+	register_bottom_half(do_abort_download, file_download);
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 dlg_undisplay_download(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct file_download *file_download = dlg_data->dlg->udata;
 
 	object_unlock(file_download);
-	register_bottom_half((void (*)(void *)) undisplay_download,
-			     file_download);
+	register_bottom_half(undisplay_download, file_download);
 	return EVENT_PROCESSED;
 }
 
@@ -103,53 +114,6 @@ download_abort_function(struct dialog_data *dlg_data)
 	file_download->dlg_data = NULL;
 }
 
-void
-download_progress_bar(struct terminal *term, int x, int y, int width,
-		      unsigned char *text, struct color_pair *meter_color,
-		      longlong current, longlong total)
-{
-	/* Note : values > 100% are theorically possible and were seen. */
-	int progress = (int) ((longlong) 100 * current / total);
-	struct box barprogress;
-
-	/* Draw the progress meter part "[###    ]" */
-	if (!text && width > 2) {
-		width -= 2;
-		draw_text(term, x++, y, "[", 1, 0, NULL);
-		draw_text(term, x + width, y, "]", 1, 0, NULL);
-	}
-
-	if (!meter_color) meter_color = get_bfu_color(term, "dialog.meter");
-	set_box(&barprogress,
-		x, y, int_min(width * progress / 100, width), 1);
-	draw_box(term, &barprogress, ' ', 0, meter_color);
-
-	/* On error, will print '?' only, should not occur. */
-	if (text) {
-		width = int_min(width, strlen(text));
-
-	} else if (width > 1) {
-		static unsigned char percent[] = "????"; /* Reduce or enlarge at will. */
-		unsigned int percent_len = 0;
-		int max = int_min(sizeof(percent), width) - 1;
-
-		if (ulongcat(percent, &percent_len, progress, max, 0)) {
-			percent[0] = '?';
-			percent_len = 1;
-		}
-
-		percent[percent_len++] = '%';
-
-		/* Draw the percentage centered in the progress meter */
-		x += (1 + width - percent_len) / 2;
-
-		assert(percent_len <= width);
-		width = percent_len;
-		text = percent;
-	}
-
-	draw_text(term, x, y, text, width, 0, NULL);
-}
 
 static void
 download_dialog_layouter(struct dialog_data *dlg_data)
@@ -166,6 +130,10 @@ download_dialog_layouter(struct dialog_data *dlg_data)
 	unsigned char *msg = get_download_msg(download, term, 1, 1, "\n");
 	int show_meter = (download_is_progressing(download)
 			  && download->progress->size >= 0);
+#if CONFIG_BITTORRENT
+	int bittorrent = (file_download->uri->protocol == PROTOCOL_BITTORRENT
+			  && (show_meter || download->state == S_RESUME));
+#endif
 
 	redraw_below_window(dlg_data->win);
 	file_download->dlg_data = dlg_data;
@@ -177,6 +145,7 @@ download_dialog_layouter(struct dialog_data *dlg_data)
 		mem_free(msg);
 		return;
 	}
+	decode_uri_for_display(url);
 	url_len = strlen(url);
 
 	if (show_meter) {
@@ -188,6 +157,10 @@ download_dialog_layouter(struct dialog_data *dlg_data)
 
 	y++;
 	if (show_meter) y += 2;
+
+#if CONFIG_BITTORRENT
+	if (bittorrent) y += 2;
+#endif
 	dlg_format_text_do(NULL, msg, 0, &y, w, &rw,
 			dialog_text_color, ALIGN_LEFT);
 
@@ -209,6 +182,7 @@ download_dialog_layouter(struct dialog_data *dlg_data)
 			url[--url_len] = '.';
 		}
 	}
+
 	y = dlg_data->box.y + DIALOG_TB + 1;
 	x = dlg_data->box.x + DIALOG_LB;
 	dlg_format_text_do(term, url, x, &y, w, NULL,
@@ -216,12 +190,17 @@ download_dialog_layouter(struct dialog_data *dlg_data)
 
 	if (show_meter) {
 		y++;
-		download_progress_bar(term, x, y, w, NULL, NULL,
-				      download->progress->pos,
-				      download->progress->size);
+		draw_progress_bar(download->progress, term, x, y, w, NULL, NULL);
 		y++;
 	}
 
+#if CONFIG_BITTORRENT
+	if (bittorrent) {
+		y++;
+		draw_bittorrent_piece_progress(download, term, x, y, w, NULL, NULL);
+		y++;
+	}
+#endif
 	y++;
 	dlg_format_text_do(term, msg, x, &y, w, NULL,
 			dialog_text_color, ALIGN_LEFT);
@@ -244,7 +223,12 @@ display_download(struct terminal *term, struct file_download *file_download,
 	if (!is_in_downloads_list(file_download))
 		return;
 
+#if CONFIG_BITTORRENT
+#define DOWNLOAD_WIDGETS_COUNT 5
+#else
 #define DOWNLOAD_WIDGETS_COUNT 4
+#endif
+
 	dlg = calloc_dialog(DOWNLOAD_WIDGETS_COUNT, 0);
 	if (!dlg) return;
 
@@ -259,17 +243,27 @@ display_download(struct terminal *term, struct file_download *file_download,
 
 	add_dlg_button(dlg, _("~Background", term), B_ENTER | B_ESC, dlg_undisplay_download, NULL);
 	add_dlg_button(dlg, _("Background with ~notify", term), B_ENTER | B_ESC, dlg_set_notify, NULL);
+
+#if CONFIG_BITTORRENT
+	if (file_download->uri->protocol == PROTOCOL_BITTORRENT)
+		add_dlg_button(dlg, _("~Info", term), B_ENTER | B_ESC, dlg_show_bittorrent_info, NULL);
+#endif
+
 	add_dlg_button(dlg, _("~Abort", term), 0, dlg_abort_download, NULL);
 
 	/* Downloads scheduled to be opened by external handlers are always
 	 * deleted. */
 	if (!file_download->external_handler) {
 		add_dlg_button(dlg, _("Abort and ~delete file", term), 0, push_delete_button, NULL);
-		add_dlg_end(dlg, DOWNLOAD_WIDGETS_COUNT);
-	} else {
-		add_dlg_end(dlg, DOWNLOAD_WIDGETS_COUNT - 1);
 	}
 
+#if CONFIG_BITTORRENT
+	add_dlg_end(dlg, DOWNLOAD_WIDGETS_COUNT - !!file_download->external_handler
+		         - (file_download->uri->protocol != PROTOCOL_BITTORRENT));
+#else
+	add_dlg_end(dlg, DOWNLOAD_WIDGETS_COUNT - !!file_download->external_handler);
+#endif
+
 	do_dialog(term, dlg, getml(dlg, NULL));
 }
 
@@ -298,8 +292,12 @@ static unsigned char *
 get_file_download_text(struct listbox_item *item, struct terminal *term)
 {
 	struct file_download *file_download = item->udata;
+	unsigned char *uristring;
 
-	return get_uri_string(file_download->uri, URI_PUBLIC);
+	uristring = get_uri_string(file_download->uri, URI_PUBLIC);
+	if (uristring) decode_uri_for_display(uristring);
+
+	return uristring;
 }
 
 static unsigned char *
@@ -334,8 +332,7 @@ delete_file_download(struct listbox_item *item, int last)
 	struct file_download *file_download = item->udata;
 
 	assert(!is_object_used(file_download));
-	register_bottom_half((void (*)(void *)) do_abort_download,
-			     file_download);
+	register_bottom_half(do_abort_download, file_download);
 }
 
 static enum dlg_refresh_code
@@ -357,8 +354,8 @@ draw_file_download(struct listbox_item *item, struct listbox_context *context,
 	struct download *download = &file_download->download;
 	unsigned char *stylename;
 	struct color_pair *color;
-	unsigned char *text = struri(file_download->uri);
-	int length = strlen(text);
+	unsigned char *text;
+	int length;
 	int trimmedlen;
 	int meter = DOWNLOAD_METER_WIDTH;
 
@@ -371,6 +368,10 @@ draw_file_download(struct listbox_item *item, struct listbox_context *context,
 
 	color = get_bfu_color(context->term, stylename);
 
+	text = get_file_download_text(item, NULL);
+	if (!text) return;
+
+	length = strlen(text);
 	/* Show atleast the required percentage of the URI */
 	if (length * DOWNLOAD_URI_PERCENTAGE / 100 < width - meter - 4) {
 		trimmedlen = int_min(length, width - meter - 4);
@@ -384,10 +385,12 @@ draw_file_download(struct listbox_item *item, struct listbox_context *context,
 		trimmedlen += 3;
 	}
 
+	mem_free(text);
+
 	if (!download->progress
 	    || download->progress->size < 0
 	    || download->state != S_TRANS
-	    || !(download->progress->elapsed / 100)) {
+	    || !has_progress(download->progress)) {
 		/* TODO: Show trimmed error message. */
 		return;
 	}
@@ -399,9 +402,7 @@ draw_file_download(struct listbox_item *item, struct listbox_context *context,
 
 	x += width - meter;
 
-	download_progress_bar(context->term, x, y, meter, NULL, NULL,
-			      download->progress->pos,
-			      download->progress->size);
+	draw_progress_bar(download->progress, context->term, x, y, meter, NULL, NULL);
 }
 
 static struct listbox_ops_messages download_messages = {
@@ -447,7 +448,7 @@ static struct listbox_ops downloads_listbox_ops = {
 };
 
 
-static t_handler_event_status
+static widget_handler_status_T
 push_info_button(struct dialog_data *dlg_data, struct widget_data *button)
 {
 	struct listbox_data *box = get_dlg_listbox_data(dlg_data);
@@ -483,7 +484,7 @@ static struct hierbox_browser_button download_buttons[] = {
 	{ N_("C~lear"),                 push_hierbox_clear_button  },
 };
 
-struct_hierbox_browser(
+static struct_hierbox_browser(
 	download_browser,
 	N_("Download manager"),
 	download_buttons,
@@ -498,3 +499,20 @@ download_manager(struct session *ses)
 	/* FIXME: It's workaround for bug 397. Real fix is needed. */
 	download_browser.do_not_save_state = 1;
 }
+
+void
+init_download_display(struct file_download *file_download)
+{
+	file_download->box_item = add_listbox_leaf(&download_browser, NULL,
+						   file_download);
+}
+
+void
+done_download_display(struct file_download *file_download)
+{
+	if (file_download->box_item) {
+		done_listbox_item(&download_browser, file_download->box_item);
+		file_download->box_item = NULL;
+	}
+}
+
