@@ -49,7 +49,7 @@ struct keepalive_connection {
 	timeval_T timeout;
 	timeval_T creation_time;
 
-	unsigned int protocol_family:1; /* 0 == PF_INET, 1 == PF_INET6 */
+	unsigned int protocol_family:1; /* see network/socket.h, EL_PF_INET, EL_PF_INET6 */
 	int socket;
 };
 
@@ -847,7 +847,7 @@ load_uri(struct uri *uri, struct uri *referrer, struct download *download,
 	struct cache_entry *cached;
 	struct connection *conn;
 	struct uri *proxy_uri, *proxied_uri;
-	enum connection_state connection_state = S_OK;
+	int connection_state = S_OK;
 
 	if (download) {
 		download->conn = NULL;
