@@ -279,7 +279,7 @@ get_pasv_socket(struct socket *ctrl_socket, struct sockaddr_storage *addr)
 #ifdef CONFIG_IPV6
 	struct sockaddr_in6 bind_addr6;
 
-	if (ctrl_socket->protocol_family == 1) {
+	if (ctrl_socket->protocol_family == EL_PF_INET6) {
 		bind_addr = (struct sockaddr *) &bind_addr6;
 		addrlen   = sizeof(bind_addr6);
 	} else
@@ -303,7 +303,7 @@ sock_error:
 
 	/* Get a passive socket */
 
-	sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (sock < 0)
 		goto sock_error;
 
@@ -316,7 +316,7 @@ sock_error:
 
 	memcpy(bind_addr, pasv_addr, addrlen);
 #ifdef CONFIG_IPV6
-	if (ctrl_socket->protocol_family == 1)
+	if (ctrl_socket->protocol_family == EL_PF_INET6)
 		bind_addr6.sin6_port = 0;
 	else
 #endif
@@ -504,6 +504,7 @@ connect_socket(struct socket *csocket, enum connection_state state)
 #else
 		struct sockaddr_in addr = *((struct sockaddr_in *) &connect_info->addr[i]);
 #endif
+		int pf;
 		int family;
 		int force_family = connect_info->ip_family;
 
@@ -532,18 +533,30 @@ connect_socket(struct socket *csocket, enum connection_state state)
 		}
 
 #ifdef CONFIG_IPV6
-		if (family == AF_INET6 && (!get_opt_bool("connection.try_ipv6") || (force_family && force_family != 6))) {
-			silent_fail = 1;
-			continue;
+		if (family == AF_INET6) {
+			pf = PF_INET6;
+			if (!get_opt_bool("connection.try_ipv6")
+			    || (force_family && force_family != 6)) {
+				silent_fail = 1;
+				continue;
+			}
+
 		} else
 #endif
-		if (family == AF_INET && (!get_opt_bool("connection.try_ipv4") || (force_family && force_family != 4))) {
-			silent_fail = 1;
+		if (family == AF_INET) {
+			pf = PF_INET;
+			if (!get_opt_bool("connection.try_ipv4")
+			    || (force_family && force_family != 4)) {
+				silent_fail = 1;
+				continue;
+			}
+
+		} else {
 			continue;
 		}
 		silent_fail = 0;
 
-		sock = socket(family, SOCK_STREAM, IPPROTO_TCP);
+		sock = socket(pf, SOCK_STREAM, IPPROTO_TCP);
 		if (sock == -1) {
 			if (errno && !saved_errno) saved_errno = errno;
 			continue;
@@ -572,7 +585,7 @@ connect_socket(struct socket *csocket, enum connection_state state)
 			if (connect(sock, (struct sockaddr *) &addr,
 					sizeof(struct sockaddr_in6)) == 0) {
 				/* Success */
-				csocket->protocol_family = 1;
+				csocket->protocol_family = EL_PF_INET6;
 				complete_connect_socket(csocket, NULL, NULL);
 				return;
 			}
@@ -582,7 +595,7 @@ connect_socket(struct socket *csocket, enum connection_state state)
 			if (connect(sock, (struct sockaddr *) &addr,
 					sizeof(struct sockaddr_in)) == 0) {
 				/* Success */
-				csocket->protocol_family = 0;
+				csocket->protocol_family = EL_PF_INET;
 				complete_connect_socket(csocket, NULL, NULL);
 				return;
 			}
