@@ -12,13 +12,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h> /* FreeBSD needs this before resource.h */
-#endif
-#include <sys/types.h> /* FreeBSD needs this before resource.h */
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
+#include <sys/types.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h> /* OS/2 needs this after sys/types.h */
 #endif
@@ -36,6 +30,7 @@
 #include "network/connection.h"
 #include "network/socket.h"
 #include "osdep/osdep.h"
+#include "protocol/common.h"
 #include "protocol/protocol.h"
 #include "protocol/smb/smb.h"
 #include "protocol/uri.h"
@@ -525,22 +520,6 @@ bye:
 }
 
 
-/* Close all non-terminal file descriptors. */
-static void
-close_all_non_term_fd(void)
-{
-	int n;
-	int max = 1024;
-#ifdef RLIMIT_NOFILE
-	struct rlimit lim;
-
-	if (!getrlimit(RLIMIT_NOFILE, &lim))
-		max = lim.rlim_max;
-#endif
-	for (n = 3; n < max; n++)
-		close(n);
-}
-
 void
 smb_protocol_handler(struct connection *conn)
 {
