@@ -1,5 +1,4 @@
 /* Lua interface (scripting engine) */
-/* $Id: core.c,v 1.173.4.11 2005/04/07 13:18:31 adamg Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -20,26 +19,26 @@
 
 #include "bfu/dialog.h"
 #include "cache/cache.h"
+#include "config/home.h"
 #include "config/kbdbind.h"
 #include "config/options.h"
 #include "config/opttypes.h"
 #include "document/document.h"
 #include "document/renderer.h"
 #include "document/view.h"
-#include "intl/gettext/libintl.h"
 #include "intl/charsets.h"
-#include "lowlevel/home.h"
-#include "lowlevel/signals.h"
-#include "modules/module.h"
+#include "intl/gettext/libintl.h"
+#include "main/event.h"
+#include "main/module.h"
+#include "osdep/signals.h"
 #include "protocol/uri.h"
-#include "sched/event.h"
-#include "sched/session.h"
-#include "sched/task.h"
 #include "scripting/lua/core.h"
-#include "scripting/lua/hooks.h"
-#include "scripting/scripting.h"
+#include "session/location.h"
+#include "session/session.h"
+#include "session/task.h"
 #include "terminal/terminal.h"
 #include "util/color.h"
+#include "util/conv.h"
 #include "util/file.h"
 #include "util/memory.h"
 #include "util/string.h"
@@ -60,6 +59,8 @@ static sigjmp_buf errjmp;
 #define LS	lua_State *S
 
 static void handle_standard_lua_returns(unsigned char *from);
+static void handle_ref(LS, struct session *ses, int func_ref,
+                       unsigned char *from, int num_args, int unref);
 
 
 /*
@@ -114,14 +115,7 @@ l_current_title(LS)
 		unsigned char *clean_title = stracpy(doc_view->document->title);
 
 		if (clean_title) {
-			int i = 0;
-
-			while (clean_title[i]) {
-				if (clean_title[i] < ' '
-				    || clean_title[i] == NBSP_CHAR)
-					clean_title[i] = ' ';
-				i++;
-			}
+			sanitize_title(clean_title);
 
 			lua_pushstring(S, clean_title);
 			mem_free(clean_title);
@@ -191,7 +185,7 @@ l_pipe_read(LS)
 {
 	FILE *fp;
 	unsigned char *s = NULL;
-	int len = 0;
+	size_t len = 0;
 
 	if (!lua_isstring(S, 1)) goto lua_error;
 
@@ -200,7 +194,7 @@ l_pipe_read(LS)
 
 	while (!feof(fp)) {
 		unsigned char buf[1024];
-		int l = fread(buf, 1, sizeof(buf), fp);
+		size_t l = fread(buf, 1, sizeof(buf), fp);
 
 		if (l > 0) {
 			s = mem_realloc(s, len + l);
@@ -253,17 +247,6 @@ l_tmpname(LS)
 	return 1;
 }
 
-static int
-l_enable_systems_functions(LS)
-{
-	lua_iolibopen(S);
-	lua_register(S, "pipe_read", l_pipe_read);
-	lua_register(S, "execute", l_execute);
-	lua_register(S, "tmpname", l_tmpname);
-
-	return 0;
-}
-
 /*
  * Helper to run Lua functions bound to keystrokes.
  */
@@ -273,18 +256,13 @@ run_lua_func(va_list ap, void *data)
 {
 	struct session *ses = va_arg(ap, struct session *);
 	int func_ref = (long) data;
-	int err;
 
 	if (func_ref == LUA_NOREF) {
 		alert_lua_error("key bound to nothing (internal error)");
 		return EVENT_HOOK_STATUS_NEXT;
 	}
 
-	lua_getref(L, func_ref);
-	if (prepare_lua(ses)) return EVENT_HOOK_STATUS_NEXT;
-	err = lua_call(L, 0, 2);
-	finish_lua();
-	if (!err) handle_standard_lua_returns("keyboard function");
+	handle_ref(L, ses, func_ref, "keyboard function", 0, 0);
 
 	return EVENT_HOOK_STATUS_NEXT;
 }
@@ -293,9 +271,9 @@ static int
 l_bind_key(LS)
 {
 	int ref;
-	unsigned char *err;
-	struct string event_name = NULL_STRING;
 	int event_id;
+	unsigned char *err = NULL;
+	struct string event_name = NULL_STRING;
 
 	if (!lua_isstring(S, 1) || !lua_isstring(S, 2)
 	    || !lua_isfunction(S, 3)) {
@@ -306,17 +284,24 @@ l_bind_key(LS)
 	if (!init_string(&event_name)) goto lua_error;
 
 	lua_pushvalue(S, 3);
-	ref = lua_ref(S, 1);
+	ref = luaL_ref(S, LUA_REGISTRYINDEX);
 	add_format_to_string(&event_name, "lua-run-func %i", ref);
-	event_id = register_event(event_name.source);
-	event_id = register_event_hook(event_id, run_lua_func, 0, (void *) (long) ref);
-	done_string(&event_name);
-	if (event_id == EVENT_NONE) goto lua_error;
 
-	err = bind_scripting_func((unsigned char *) lua_tostring(S, 1),
-			    (unsigned char *) lua_tostring(S, 2), event_id);
+	event_id = bind_key_to_event_name((unsigned char *) lua_tostring(S, 1),
+					  (unsigned char *) lua_tostring(S, 2),
+					  event_name.source, &err);
+	done_string(&event_name);
+
+	if (!err) {
+		event_id = register_event_hook(event_id, run_lua_func, 0,
+					       (void *) (long) ref);
+
+		if (event_id == EVENT_NONE)
+			err = gettext("Error registering event hook");
+	}
+
 	if (err) {
-		lua_unref(S, ref);
+		luaL_unref(S, LUA_REGISTRYINDEX, ref);
 		alert_lua_error2("error in bind_key: ", err);
 		goto lua_error;
 	}
@@ -346,17 +331,11 @@ dialog_run_lua(void *data_)
 {
 	struct lua_dlg_data *data = data_;
 	lua_State *s = data->state;
-	int err;
 
-	lua_getref(s, data->func_ref);
 	lua_pushstring(s, data->cat);
 	lua_pushstring(s, data->name);
 	lua_pushstring(s, data->url);
-	if (prepare_lua(lua_ses)) return;
-	err = lua_call(s, 3, 2);
-	finish_lua();
-	lua_unref(s, data->func_ref);
-	handle_standard_lua_returns("post dialog function");
+	handle_ref(s, lua_ses, data->func_ref, "post dialog function", 3, 1);
 }
 
 static int
@@ -385,7 +364,7 @@ l_edit_bookmark_dialog(LS)
 	safe_strncpy(data->url, (unsigned char *) lua_tostring(S, 3),
 		     MAX_STR_LEN-1);
 	lua_pushvalue(S, 4);
-	data->func_ref = lua_ref(S, 1);
+	data->func_ref = luaL_ref(S, LUA_REGISTRYINDEX);
 
 	dlg->title = _("Edit bookmark", term);
 	dlg->layouter = generic_dialog_layouter;
@@ -426,27 +405,26 @@ xdialog_run_lua(void *data_)
 {
 	struct lua_xdialog_data *data = data_;
 	lua_State *s = data->state;
-	int err;
 	int i;
 
-	lua_getref(s, data->func_ref);
 	for (i = 0; i < data->nfields; i++) lua_pushstring(s, data->fields[i]);
-	if (prepare_lua(lua_ses)) return;
-	err = lua_call(s, data->nfields, 2);
-	finish_lua();
-	lua_unref(s, data->func_ref);
-	handle_standard_lua_returns("post xdialog function");
+	handle_ref(s, lua_ses, data->func_ref, "post xdialog function",
+	           data->nfields, 1);
 }
 
 static int
 l_xdialog(LS)
 {
-	struct terminal *term = lua_ses->tab->term;
+	struct terminal *term;
 	struct dialog *dlg;
 	struct lua_xdialog_data *data;
 	int nargs, nfields, nitems;
 	int i = 0;
 
+	if (!lua_ses) return 0;
+
+	term = lua_ses->tab->term;
+
 	nargs = lua_gettop(S);
 	nfields = nargs - 1;
 	nitems = nfields + 2;
@@ -466,7 +444,7 @@ l_xdialog(LS)
 			     (unsigned char *) lua_tostring(S, i+1),
 			     MAX_STR_LEN-1);
 	lua_pushvalue(S, nargs);
-	data->func_ref = lua_ref(S, 1);
+	data->func_ref = luaL_ref(S, LUA_REGISTRYINDEX);
 
 	dlg->title = _("User dialog", term);
 	dlg->layouter = generic_dialog_layouter;
@@ -515,6 +493,14 @@ l_set_option(LS)
 	/* Set option */
 	switch (opt->type) {
 	case OPT_BOOL:
+	{
+		int value;
+
+		value = lua_toboolean(S, 2);
+		option_types[opt->type].set(opt, (unsigned char *) (&value));
+		break;
+	}
+
 	case OPT_INT:
 	case OPT_LONG:
 	{
@@ -534,15 +520,11 @@ l_set_option(LS)
 		goto lua_error;
 	}
 
+	opt->flags |= OPT_TOUCHED;
+
 	/* Call hook */
 	current = opt;
-	while (current && (!current->change_hook ||
-		!current->change_hook(lua_ses, current, opt))) {
-		if (current->root)
-			current = current->root;
-		else
-			break;
-	}
+	call_change_hooks(lua_ses, current, opt);
 	return 1;
 
 lua_error:
@@ -569,10 +551,8 @@ l_get_option(LS)
 	/* Convert to an appropriate Lua type */
 	switch (opt->type) {
 	case OPT_BOOL:
-#ifdef HAVE_LUA5
 		lua_pushboolean(S, opt->value.number);
 		break;
-#endif
 	case OPT_INT:
 	case OPT_LONG:
 		lua_pushnumber(S, opt->value.number);
@@ -602,7 +582,7 @@ l_get_option(LS)
 	}
 	case OPT_COLOR:
 	{
-		color_t color;
+		color_T color;
 		unsigned char hexcolor[8];
 		unsigned char *strcolor;
 
@@ -625,6 +605,20 @@ lua_error:
 
 /* End of set/get option */
 
+int
+eval_function(LS, int num_args, int num_results)
+{
+	int err;
+
+	err = lua_pcall(S, num_args, num_results, 0);
+	if (err) {
+		alert_lua_error((unsigned char *) lua_tostring(L, -1));
+		lua_pop(L, 1);
+	}
+
+	return err;
+}
+
 /* Initialisation */
 
 static void
@@ -647,31 +641,35 @@ do_hooks_file(LS, unsigned char *prefix, unsigned char *filename)
 	mem_free(file);
 }
 
-static void
+void
 init_lua(struct module *module)
 {
-	L = lua_open(0);
-	lua_baselibopen(L);
-	lua_mathlibopen(L);
-	lua_strlibopen(L);
+	L = lua_open();
+
+	luaopen_base(L);
+	luaopen_table(L);
+	luaopen_io(L);
+	luaopen_string(L);
+	luaopen_math(L);
+
 	lua_register(L, LUA_ALERT, l_alert);
 	lua_register(L, "current_url", l_current_url);
 	lua_register(L, "current_link", l_current_link);
 	lua_register(L, "current_title", l_current_title);
 	lua_register(L, "current_document", l_current_document);
 	lua_register(L, "current_document_formatted", l_current_document_formatted);
-	lua_register(L, "enable_systems_functions", l_enable_systems_functions);
+	lua_register(L, "pipe_read", l_pipe_read);
+	lua_register(L, "execute", l_execute);
+	lua_register(L, "tmpname", l_tmpname);
 	lua_register(L, "bind_key", l_bind_key);
 	lua_register(L, "edit_bookmark_dialog", l_edit_bookmark_dialog);
 	lua_register(L, "xdialog", l_xdialog);
 	lua_register(L, "set_option", l_set_option);
 	lua_register(L, "get_option", l_get_option);
 
-	lua_dostring(L, "function set_elinks_home(s) elinks_home = s end");
-	lua_getglobal(L, "set_elinks_home");
 	lua_pushstring(L, elinks_home ? elinks_home
 				      : (unsigned char *) CONFDIR);
-	lua_call(L, 1, 0);
+	lua_setglobal(L, "elinks_home");
 
 	do_hooks_file(L, CONFDIR, LUA_HOOKS_FILENAME);
 	if (elinks_home) do_hooks_file(L, elinks_home, LUA_HOOKS_FILENAME);
@@ -679,7 +677,7 @@ init_lua(struct module *module)
 
 static void free_lua_console_history_entries(void);
 
-static void
+void
 cleanup_lua(struct module *module)
 {
 	free_lua_console_history_entries();
@@ -722,9 +720,9 @@ void
 alert_lua_error(unsigned char *msg)
 {
 	if (errterm) {
-		info_box(errterm, MSGBOX_NO_TEXT_INTL,
+		info_box(errterm, MSGBOX_NO_TEXT_INTL | MSGBOX_FREE_TEXT,
 			N_("Lua Error"), ALIGN_LEFT,
-			msg);
+			stracpy(msg));
 		return;
 	}
 
@@ -814,6 +812,33 @@ handle_standard_lua_returns(unsigned char *from)
 	lua_pop(L, 2);
 }
 
+static void
+handle_ref_on_stack(LS, struct session *ses, unsigned char *from, int num_args)
+{
+	int err;
+
+	if (prepare_lua(ses)) return;
+	err = eval_function(S, num_args, 2);
+	finish_lua();
+
+	if (!err) handle_standard_lua_returns(from);
+}
+
+static void
+handle_ref(LS, struct session *ses, int func_ref, unsigned char *from,
+           int num_args, int unref)
+{
+	lua_rawgeti(S, LUA_REGISTRYINDEX, func_ref);
+
+	/* The function must be below the arguments on the stack. */
+	if (num_args != 0) lua_insert(S, -(num_args + 1));
+
+	handle_ref_on_stack(S, ses, from, num_args);
+
+	if (unref)
+		luaL_unref(S, LUA_REGISTRYINDEX, func_ref);
+}
+
 
 /* Console stuff. */
 
@@ -830,14 +855,13 @@ lua_console(struct session *ses, unsigned char *expr)
 	}
 
 	lua_pushstring(L, expr);
-	if (prepare_lua(ses) == 0) {
-		int err = lua_call(L, 1, 2);
-
-		finish_lua();
-		if (!err) handle_standard_lua_returns("lua_console_hook");
-	}
+	handle_ref_on_stack(L, ses, "lua_console_hook", 1);
 }
 
+/* TODO: Make this a "Scripting console" instead, with a radiobutton below the
+ * inputbox selecting the appropriate scripting backend to use to evaluate the
+ * expression. --pasky */
+
 enum evhook_status
 dialog_lua_console(va_list ap, void *data)
 {
@@ -866,14 +890,3 @@ free_lua_console_history(va_list ap, void *data)
 	free_lua_console_history_entries();
 	return EVENT_HOOK_STATUS_NEXT;
 }
-
-
-struct module lua_scripting_module = struct_module(
-	/* name: */		"Lua",
-	/* options: */		NULL,
-	/* hooks: */		lua_scripting_hooks,
-	/* submodules: */	NULL,
-	/* data: */		NULL,
-	/* init: */		init_lua,
-	/* done: */		cleanup_lua
-);
