@@ -1,5 +1,4 @@
 /* Textarea form item handlers */
-/* $Id: textarea.c,v 1.143.2.2 2005/05/01 22:03:23 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -25,13 +24,14 @@
 #include "document/html/renderer.h"
 #include "document/view.h"
 #include "intl/gettext/libintl.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/window.h"
 #include "util/error.h"
 #include "util/file.h"
 #include "util/memory.h"
 #include "util/string.h"
+#include "viewer/action.h"
 #include "viewer/text/form.h"
 #include "viewer/text/textarea.h"
 #include "viewer/text/view.h"
@@ -311,42 +311,11 @@ save_textarea_file(unsigned char *value)
 	return filename;
 }
 
-static unsigned char *
-load_textarea_file(unsigned char *filename, int maxlength)
-{
-	unsigned char *value = NULL;
-	FILE *file = fopen(filename, "rb+");
-	int filelen = -1;
-
-	if (!file) return NULL;
-
-	if (!fseek(file, 0, SEEK_END)) {
-		filelen = ftell(file);
-		if (filelen != -1 && fseek(file, 0, SEEK_SET))
-			filelen = -1;
-	}
-
-	if (filelen >= 0 && filelen <= maxlength) {
-		int bread;
-
-		value = mem_alloc(filelen + 1);
-		if (value) {
-			bread = fread(value, 1, filelen, file);
-			value[bread] = 0;
-		}
-	}
-
-	fclose(file);
-	unlink(filename);
-
-	return value;
-}
-
 void
 textarea_edit(int op, struct terminal *term_, struct form_state *fs_,
 	      struct document_view *doc_view_, struct link *link_)
 {
-	static int fc_maxlength;
+	static size_t fc_maxlength;
 	static struct form_state *fs;
 	static struct terminal *term;
 	static struct document_view *doc_view;
@@ -372,13 +341,16 @@ textarea_edit(int op, struct terminal *term_, struct form_state *fs_,
 	}
 
 	if (op == 0 && !textarea_editor) {
-		unsigned char *ed = getenv("EDITOR");
+		unsigned char *ed = get_opt_str("document.browse.forms.editor");
 		unsigned char *ex;
 
 		fn = save_textarea_file(fs_->value);
 		if (!fn) return;
 
-		if (!ed || !*ed) ed = "vi";
+		if (!ed || !*ed) {
+			ed = getenv("EDITOR");
+			if (!ed || !*ed) ed = "vi";
+		}
 
 		ex = straconcat(ed, " ", fn, NULL);
 		if (!ex) {
@@ -400,17 +372,39 @@ textarea_edit(int op, struct terminal *term_, struct form_state *fs_,
 		textarea_editor = 1;
 
 	} else if (op == 1 && fs) {
-		unsigned char *value = load_textarea_file(fn, fc_maxlength);
+		struct string file;
 
-		if (value) {
-			mem_free(fs->value);
-			fs->value = value;
-			fs->state = strlen(value);
-
-			if (doc_view && link)
-				draw_form_entry(term, doc_view, link);
+		if (!init_string(&file)
+		     || !add_file_to_string(&file, fn)) {
+			textarea_editor = 0;
+			goto free_and_return;
 		}
 
+		if (file.length > fc_maxlength) {
+			file.source[fc_maxlength] = '\0';
+			info_box(term, MSGBOX_FREE_TEXT, N_("Warning"),
+			         ALIGN_CENTER,
+			         msg_text(term,
+				          N_("You have exceeded the textarea's"
+				             " size limit: your input is %d"
+					     " bytes, but the maximum is %u"
+					     " bytes.\n\n"
+					     "Your input has been truncated,"
+					     " but you can still recover the"
+					     " text that you entered from"
+					     " this file: %s"), file.length,
+				             fc_maxlength, fn));
+		} else {
+			unlink(fn);
+		}
+
+		mem_free(fs->value);
+		fs->value = file.source;
+		fs->state = file.length;
+
+		if (doc_view && link)
+			draw_form_entry(term, doc_view, link);
+
 		textarea_editor = 0;
 		goto free_and_return;
 	}
@@ -422,7 +416,7 @@ free_and_return:
 	fs = NULL;
 }
 
-/* menu_func */
+/* menu_func_T */
 void
 menu_textarea_edit(struct terminal *term, void *xxx, void *ses_)
 {
@@ -453,11 +447,9 @@ menu_textarea_edit(struct terminal *term, void *xxx, void *ses_)
 	textarea_edit(0, term, fs, doc_view, link);
 }
 
-
-/* TODO: Unify the textarea field_op handlers to one trampoline function. */
-
-enum frame_event_status
-textarea_op_home(struct form_state *fs, struct form_control *fc)
+static enum frame_event_status
+textarea_op(struct form_state *fs, struct form_control *fc,
+	    int (*do_op)(struct form_state *, struct line_info *, int))
 {
 	struct line_info *line;
 	int current, state;
@@ -465,95 +457,53 @@ textarea_op_home(struct form_state *fs, struct form_control *fc)
 	assert(fs && fs->value && fc);
 	if_assert_failed return FRAME_EVENT_OK;
 
-	state = fs->state;
 	line = format_text(fs->value, fc->cols, fc->wrap, 0);
 	if (!line) return FRAME_EVENT_OK;
 
 	current = get_textarea_line_number(line, fs->state);
-	if (current != -1) fs->state = line[current].start;
+	state = fs->state;
+	if (do_op(fs, line, current)) {
+		mem_free(line);
+		return FRAME_EVENT_IGNORED;
+	}
 
 	mem_free(line);
 
 	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
 }
 
-enum frame_event_status
-textarea_op_up(struct form_state *fs, struct form_control *fc)
+static int
+do_op_home(struct form_state *fs, struct line_info *line, int current)
 {
-	struct line_info *line;
-	int current, state;
+	if (current != -1) fs->state = line[current].start;
+	return 0;
+}
 
-	assert(fs && fs->value && fc);
-	if_assert_failed return FRAME_EVENT_OK;
+static int
+do_op_up(struct form_state *fs, struct line_info *line, int current)
+{
+	if (current == -1) return 0;
+	if (!current) return 1;
 
-	line = format_text(fs->value, fc->cols, fc->wrap, 0);
-	if (!line) return FRAME_EVENT_OK;
-
-
-	current = get_textarea_line_number(line, fs->state);
-	if (current == -1) {
-		mem_free(line);
-		return FRAME_EVENT_OK;
-	}
-
-	if (!current) {
-		mem_free(line);
-		return FRAME_EVENT_IGNORED;
-	}
-
-	state = fs->state;
 	fs->state -= line[current].start - line[current-1].start;
 	int_upper_bound(&fs->state, line[current-1].end);
-
-	mem_free(line);
-	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
+	return 0;
 }
 
-enum frame_event_status
-textarea_op_down(struct form_state *fs, struct form_control *fc)
+static int
+do_op_down(struct form_state *fs, struct line_info *line, int current)
 {
-	struct line_info *line;
-	int current, state;
+	if (current == -1) return 0;
+	if (line[current+1].start == -1) return 1;
 
-	assert(fs && fs->value && fc);
-	if_assert_failed return FRAME_EVENT_OK;
-
-	line = format_text(fs->value, fc->cols, fc->wrap, 0);
-	if (!line) return FRAME_EVENT_OK;
-
-	current = get_textarea_line_number(line, fs->state);
-	if (current == -1) {
-		mem_free(line);
-		return FRAME_EVENT_OK;
-	}
-
-	if (line[current+1].start == -1) {
-		mem_free(line);
-		return FRAME_EVENT_IGNORED;
-	}
-
-	state = fs->state;
 	fs->state += line[current+1].start - line[current].start;
 	int_upper_bound(&fs->state, line[current+1].end);
-
-	mem_free(line);
-	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
+	return 0;
 }
 
-enum frame_event_status
-textarea_op_end(struct form_state *fs, struct form_control *fc)
+static int
+do_op_end(struct form_state *fs, struct line_info *line, int current)
 {
-	struct line_info *line;
-	int current, state;
-
-	assert(fs && fs->value && fc);
-	if_assert_failed return FRAME_EVENT_OK;
-
-	state = fs->state;
-	line = format_text(fs->value, fc->cols, fc->wrap, 0);
-	if (!line) return FRAME_EVENT_OK;
-
-	current = get_textarea_line_number(line, fs->state);
 	if (current == -1) {
 		fs->state = strlen(fs->value);
 
@@ -563,54 +513,22 @@ textarea_op_end(struct form_state *fs, struct form_control *fc)
 		/* Don't jump to next line when wrapping. */
 		fs->state = int_max(0, line[current].end - wrap);
 	}
-
-	mem_free(line);
-	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
+	return 0;
 }
 
-/* Set the form state so the cursor is on the first line of the buffer.
- * Preserve the column if possible. */
-enum frame_event_status
-textarea_op_bob(struct form_state *fs, struct form_control *fc)
+static int
+do_op_bob(struct form_state *fs, struct line_info *line, int current)
 {
-	struct line_info *line;
-	int current, state;
+	if (current == -1) return 0;
 
-	assert(fs && fs->value && fc);
-	if_assert_failed return FRAME_EVENT_OK;
-
-	state = fs->state;
-	line = format_text(fs->value, fc->cols, fc->wrap, 0);
-	if (!line) return FRAME_EVENT_OK;
-
-	current = get_textarea_line_number(line, fs->state);
-	if (current != -1) {
-		fs->state -= line[current].start;
-		int_upper_bound(&fs->state, line[0].end);
-	}
-
-	mem_free(line);
-	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
+	fs->state -= line[current].start;
+	int_upper_bound(&fs->state, line[0].end);
+	return 0;
 }
 
-/* Set the form state so the cursor is on the last line of the buffer. Preserve
- * the column if possible. This is done by getting current and last line and
- * then shifting the state by the delta of both lines start position bounding
- * the whole thing to the end of the last line. */
-enum frame_event_status
-textarea_op_eob(struct form_state *fs, struct form_control *fc)
+static int
+do_op_eob(struct form_state *fs, struct line_info *line, int current)
 {
-	struct line_info *line;
-	int current, state;
-
-	assert(fs && fs->value && fc);
-	if_assert_failed return FRAME_EVENT_OK;
-
-	state = fs->state;
-	line = format_text(fs->value, fc->cols, fc->wrap, 0);
-	if (!line) return FRAME_EVENT_OK;
-
-	current = get_textarea_line_number(line, fs->state);
 	if (current == -1) {
 		fs->state = strlen(fs->value);
 
@@ -622,9 +540,49 @@ textarea_op_eob(struct form_state *fs, struct form_control *fc)
 		fs->state += line[last].start - line[current].start;
 		int_upper_bound(&fs->state, line[last].end);
 	}
+	return 0;
+}
 
-	mem_free(line);
-	return fs->state == state ? FRAME_EVENT_OK : FRAME_EVENT_REFRESH;
+enum frame_event_status
+textarea_op_home(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_home);
+}
+
+enum frame_event_status
+textarea_op_up(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_up);
+}
+
+enum frame_event_status
+textarea_op_down(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_down);
+}
+
+enum frame_event_status
+textarea_op_end(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_end);
+}
+
+/* Set the form state so the cursor is on the first line of the buffer.
+ * Preserve the column if possible. */
+enum frame_event_status
+textarea_op_bob(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_bob);
+}
+
+/* Set the form state so the cursor is on the last line of the buffer. Preserve
+ * the column if possible. This is done by getting current and last line and
+ * then shifting the state by the delta of both lines start position bounding
+ * the whole thing to the end of the last line. */
+enum frame_event_status
+textarea_op_eob(struct form_state *fs, struct form_control *fc)
+{
+	return textarea_op(fs, fc, do_op_eob);
 }
 
 enum frame_event_status
